@@ -6,7 +6,7 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'blessed', 'weaken', 'looks_like_number', 'refaddr';
 
-our $VERSION   = '0.97_01';
+our $VERSION   = '0.83';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -34,22 +34,6 @@ sub can_be_inlined {
     my $metaclass = $self->associated_metaclass;
     my $class     = $metaclass->name;
 
-    # If we don't find an inherited method, this is a rather weird
-    # case where we have no method in the inheritance chain even
-    # though we're expecting one to be there
-    my $inherited_method
-        = $metaclass->find_next_method_by_name( $self->name );
-
-    if (   $inherited_method
-        && $inherited_method->isa('Class::MOP::Method::Wrapped') ) {
-        warn "Not inlining '"
-            . $self->name
-            . "' for $class since it "
-            . "has method modifiers which would be lost if it were inlined\n";
-
-        return 0;
-    }
-
     my $expected_class = $self->_expected_method_class
         or return 1;
 
@@ -74,11 +58,19 @@ sub can_be_inlined {
     return 1
         if refaddr($expected_method) == refaddr($actual_method);
 
+    # If we don't find an inherited method, this is a rather weird
+    # case where we have no method in the inheritance chain even
+    # though we're expecting one to be there
+    #
+    # this returns 1 for backwards compatibility for now
+    my $inherited_method
+        = $metaclass->find_next_method_by_name( $self->name )
+            or return 1;
+
     # otherwise we have to check that the actual method is an inlined
     # version of what we're expecting
     if ( $inherited_method->isa(__PACKAGE__) ) {
-        if ( $inherited_method->_uninlined_body
-             && refaddr( $inherited_method->_uninlined_body )
+        if ( refaddr( $inherited_method->_uninlined_body )
              == refaddr($expected_method) ) {
             return 1;
         }
@@ -104,6 +96,12 @@ sub can_be_inlined {
             . " call to $class->meta->make_immutable\n";
     }
 
+    $warning
+        .= " ('"
+        . $self->name
+        . "' has method modifiers which would be lost if it were inlined)\n"
+        if $inherited_method->isa('Class::MOP::Method::Wrapped');
+
     warn $warning;
 
     return 0;
@@ -143,7 +141,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2010 by Infinity Interactive, Inc.
+Copyright 2006-2009 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
